<?php

namespace EkomiIntegration\EventListeners;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Thelia\Core\Event\ActionEvent;
use Thelia\Model\ConfigQuery;
use Thelia\Model\ProductImageQuery;
use Thelia\Model\ProductQuery;

class StatusChangeListner implements EventSubscriberInterface
{
    /**
     * Returns plugin configuration values
     * @return array
     */
    public function getConfigValues()
    {
        $config = array(
            'mode'       => ConfigQuery::read("ekomiintegration_mode",     ""),
            'shopId'     => ConfigQuery::read("ekomiintegration_shopid",   ""),
            'shopSecret' => ConfigQuery::read("ekomiintegration_secret",   ""),
            'productRev' => ConfigQuery::read("ekomiintegration_reviews",  ""),
            'statuses'   => ConfigQuery::read("ekomiintegration_statuses", ""),
            'termsAndConditions'   => ConfigQuery::read("ekomiintegration_termsAndConditions", ""),
        );
        return $config;
    }

    /**
     * Returns customer cellphone
     * @param $event
     *
     * @return mixed
     */
    public function getCellPhone($event)
    {
        $telephone = $event->getOrder()->getCustomer()->getAddresses()->getFirst()->getCellPhone();
        if(empty($telephone))
            $telephone = $event->getOrder()->getCustomer()->getAddresses()->getFirst()->getPhone();
        return $telephone;
    }

    /**
     * Validates CellPhone format
     * @param $phone
     *
     * @return bool
     */
    public function validateE164($phone)
    {
        $pattern = '/^\+?[1-9]\d{1,14}$/';
        preg_match($pattern, $phone, $matches);
        if (!empty($matches)) {
            return true;
        }
        return false;
    }

    /**
     * Determines API mode based on config value and cellphone
     * @param $configMode
     * @param $telephone
     *
     * @return string
     */
    public function getApiMode($configMode, $telephone)
    {
        $e164    = $this->validateE164($telephone);
        $apiMode = 'email';

        switch($configMode){
            case '0':
                $apiMode = 'sms';
                break;
            case '1':
                $apiMode = 'email';
                break;
            case '2':
                if($e164)
                    $apiMode = 'sms';
                else
                    $apiMode = 'email';
                break;
        }
        return $apiMode;
    }

    /**
     * Returns all the required fields for API
     * @param $event
     * @param $config
     *
     * @return array
     */
    public function getRequiredFields($event, $config)
    {
        $telephone = $this->getCellPhone($event);
        $apiMode   = $this->getApiMode($config['mode'], $telephone);
        $firstName = $event->getOrder()->getCustomer()->getFirstname();
        $lastName  = $event->getOrder()->getCustomer()->getLastname();

        $fields = array(
            'recipient_type'   => $apiMode,
            'shop_id'          => $config['shopId'],
            'password'         => $config['shopSecret'],
            'salutation'       => '',
            'first_name'       => $firstName,
            'last_name'        => $lastName,
            'email'            => $event->getOrder()->getCustomer()->getEmail(),
            'transaction_id'   => $event->getOrder()->getRef(),
            'transaction_time' => $event->getOrder()->getInvoiceDate()->format('Y-m-d H:i:s'),
            'telephone'        => $telephone,
            'sender_name'      => ConfigQuery::getStoreName(),
            'sender_email'     => ConfigQuery::getStoreEmail(),
            'client_id'        => $event->getOrder()->getCustomer()->getId(),
            'screen_name'      => $firstName . " " . $lastName
        );
        return $fields;
    }

    /**
     * Returns required products data
     * @param $event
     *
     * @return mixed
     */
    public function getProductData($event)
    {
        $baseUrl = ConfigQuery::read('url_site');
        foreach ($event->getOrder()->getOrderProducts()->getData() as $product) {
            $productsData['product_info'][$product->getProductSaleElementsId()] = urlencode($product->getTitle());
            $productObj = ProductQuery::create()->findPk($product->getProductSaleElementsId());
            $locale = $productObj->getLocale();
            $canonicalUrl = $baseUrl . DS . $productObj->getRewrittenUrl($locale);
            $productImages = ProductImageQuery::create()->findByProductId($product->getProductSaleElementsId());
            foreach ($productImages as $productImage) {
                $imgUrl = $baseUrl . "/cache/images/product/" . $productImage->getFile();
                $productOther['image_url'] = utf8_decode($imgUrl);
            }
            $productOther = array(
                'product_ids' => array(
                    'gbase' => utf8_decode($product->getProductRef())
                ), // product IDs
                'links' => array(
                    array('rel' => 'canonical', 'type' => 'text/html',
                          'href' => utf8_decode($canonicalUrl))
                )
            );
            $productsData['other'][$product->getProductSaleElementsId()]['product_other'] = $productOther;
        }
        return $productsData;
    }

    /**
     * Format fields to required format
     * @param $fields
     *
     * @return string
     */
    public function getFormatedFields($fields)
    {
        $postvars = '';
        $counter  = 1;
        foreach ($fields as $key => $value) {
            if ($counter > 1) {
                $postvars .= "&";
            }
            $postvars .= $key . "=" . $value;
            $counter++;
        }
        return $postvars;
    }

    /**
     * Sends data to SRR
     * @param $postvars
     *
     * @return mixed
     */
    public function sendData($postvars)
    {
        if ((!empty($postvars)) && ($this->getConfigValues()['termsAndConditions'])) {
            $boundary = md5(time());
            $apiUrl = 'https://srr.ekomi.com/add-recipient';
            try {
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $apiUrl);
                curl_setopt($ch, CURLOPT_HEADER, false);
                curl_setopt(
                    $ch, CURLOPT_HTTPHEADER,
                    array('ContentType:multipart/form-data;boundary=' . $boundary)
                );
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                curl_setopt($ch, CURLOPT_POST, 1);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $postvars);
                $exec = curl_exec($ch);
                curl_close($ch);

                return $exec;
            } catch (Exception $e) {

            }
        }
    }

    /**
     * Executes on order status change event
     * @param ActionEvent $event
     */
    public function create(ActionEvent $event)
    {
        $config = $this->getConfigValues();
        if ($event->getStatus() == $config['statuses']) {
            $fields = $this->getRequiredFields($event, $config);
            if ($config['productRev']) {
                $productsData = $this->getProductData($event);
                $fields['has_products'] = 1;
                $fields['products_info'] = json_encode($productsData['product_info']);
                $fields['products_other'] = json_encode($productsData['other']);
            }
            $postVars = $this->getFormatedFields($fields);
            $response = $this->sendData($postVars);
        }
    }

    /**
     * Registers create() method to execute on order status change event
     * @return array
     */
    public static function getSubscribedEvents()
    {
        return array(
            "action.order.updateStatus" => array("create", 0),
        );
    }
}